﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using EPiServer.Shell.Web.Mvc.Html;

namespace EPiServer.Shell.Search
{
    [SearchProvider]
    public class TypeSearchProvider : ISearchProvider
    {
        #region ISearchProvider Members

        public string Area
        {
            get { return "Shell"; }
        }

        public string Category
        {
            get { return "Runtime"; }
        }

        public IEnumerable<SearchResult> Search(Query query)
        {
            return AppDomain.CurrentDomain.GetAssemblies()
                .SelectMany(a => a.GetTypes())
                .Where(t => t.FullName.IndexOf(query.SearchQuery, StringComparison.OrdinalIgnoreCase) > 0)
                .Select(t => CreateResult(t))
                .Take(query.MaxResults);
        }

        private static SearchResult CreateResult(Type t)
        {
            // this code is unreadable by design
            var result = new SearchResult("#" + t.FullName, t.Name);
            foreach (var tte in typeof(Type).GetProperties()
                                .Select(pi => {
                                    try
                                    {
                                        return new ToolTipElement(pi.Name, (pi.GetValue(t, null) ?? (object)"").ToString());
                                    }
                                    catch
                                    {
                                        return null;
                                    }
                                }).Where(tte => tte != null))
            {
                result.ToolTipElements.Add(tte);
            }
            return result;
        }

        #endregion
    }
}
